package com.alcar.aegis.service.businesslogic.impl;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import oracle.ucp.UniversalConnectionPoolAdapter;
import oracle.ucp.UniversalConnectionPoolException;
import oracle.ucp.admin.UniversalConnectionPoolManager;
import oracle.ucp.admin.UniversalConnectionPoolManagerImpl;
import oracle.ucp.jdbc.PoolDataSource;
import oracle.ucp.jdbc.PoolDataSourceFactory;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.log4j.Logger;
import org.springframework.stereotype.Service;

import com.alcar.aegis.persistence.AbstractDao;
import com.alcar.aegis.persistence.model.Esquema;
import com.alcar.aegis.service.businesslogic.IConexionService;
import com.alcar.aegis.utils.Mensajes;
import com.google.common.collect.Lists;

@Service
public class ConexionService implements IConexionService {

	private Logger log = Logger.getLogger(ConexionService.class);

	@Override
	public boolean comprobarConexion(Esquema esquema) throws Exception {
		Boolean conectividad = true;
		Connection conexion = null;
		DataSource datasource = null;
		try {
			datasource = crearDatasource(esquema);
			conexion = datasource.getConnection();
			log.debug("Conexion obtenida: " + conexion);
		} catch (SQLException e) {
			conectividad = false;
			log.error(ExceptionUtils.getFullStackTrace(e));
			throw new RuntimeException(e.getMessage(), e);
		} finally {
			if (conexion != null) {
				try {
					conexion.close();
				} catch (SQLException e) {
					log.error(ExceptionUtils.getFullStackTrace(e));
				}
			}
			destruirDatasource(esquema);
		}
		return conectividad;
	}
	
	@Override
	public DataSource crearDatasource(Esquema esquema) throws SQLException, UniversalConnectionPoolException {

		log.info(Mensajes.logInicio("Creando datasource al esquema "
				+ esquema.gettNombre()));
		log.debug(esquema);
		UniversalConnectionPoolManager mgr = UniversalConnectionPoolManagerImpl.getUniversalConnectionPoolManager();
		PoolDataSource pds = PoolDataSourceFactory.getPoolDataSource();
		pds.setConnectionPoolName(esquema.gettNombre());
		pds.setConnectionFactoryClassName(esquema.gettDriverClass());
		pds.setURL(esquema.gettUrl()); 
		pds.setUser(esquema.gettUsuario());
		pds.setPassword(esquema.gettPassword());
		mgr.createConnectionPool((UniversalConnectionPoolAdapter)pds);
		log.info(Mensajes.logFin("Creando datasource al esquema "
				+ esquema.gettNombre()));
		return pds;
	}
	
	@Override
	public void destruirDatasource(Esquema esquema) throws UniversalConnectionPoolException {
		log.info(Mensajes.logInicio("Destruyendo datasource: "+ esquema.gettNombre()));
		log.debug(esquema);
		UniversalConnectionPoolManager mgr = UniversalConnectionPoolManagerImpl.getUniversalConnectionPoolManager();
		String[] poolNames = mgr.getConnectionPoolNames();
		int existPool = Arrays.asList(poolNames).indexOf(esquema.gettNombre());
		if (existPool != -1 ) {
			mgr.destroyConnectionPool(esquema.gettNombre());
		} else {
			log.warn("No existe el dataource " + esquema.gettNombre() + " en el pool");
		}
		log.info(Mensajes.logFin("Destruyendo datasource: "+ esquema.gettNombre()));
	}
	
	@Override
	public Integer probarConsulta(Esquema esquema, String consulta) throws Exception {
		Integer numFilas = 0;
		DataSource datasource = null;
		try {
			datasource = crearDatasource(esquema);
			AbstractDao dao = new AbstractDao(datasource);
			numFilas = dao.probarConsulta(consulta);
		} catch (SQLException e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			throw new RuntimeException(e.getMessage(), e);
		} finally {
			destruirDatasource(esquema);
		}
		return numFilas;
	}
	
	@Override
	public List<Map<String, Object>> ejecutarConsulta(Esquema esquema, String consulta, Map<String, Object> parametros) throws Exception {
		DataSource datasource = null;
		List<Map<String, Object>> objetos = Lists.newArrayList();
		try {
			
			datasource = crearDatasource(esquema);
			AbstractDao dao = new AbstractDao(datasource);
			objetos = dao.LanzarConsulta(consulta, parametros);
			
		} catch (SQLException e) {
			log.error(ExceptionUtils.getFullStackTrace(e));
			throw new RuntimeException(e.getMessage(), e);
		} finally {
			destruirDatasource(esquema);
		}
		return objetos;
	}
	

}
